@@ -492,7 +492,7 @@ typedef struct {
  * implementation defined.  GnuPG uses this to cache signature
  * verification status and since 2.1.18 also to convey information
  * about the origin of a key.  Note that this packet is not part
- * struct packet_struct becuase we use it only local in the packet
+ * struct packet_struct because we use it only local in the packet
  * parser and builder. */
 typedef struct {
   unsigned int trustval;
@@ -623,7 +623,8 @@ struct parse_packet_ctx_s
   iobuf_t inp;       /* The input stream with the packets.  */
   struct packet_struct last_pkt; /* The last parsed packet.  */
   int free_last_pkt; /* Indicates that LAST_PKT must be freed.  */
-  int skip_meta;     /* Skip right trust packets.  */
+  int skip_meta;     /* Skip ring trust packets.  */
+  unsigned int n_parsed_packets;	/* Number of parsed packets.  */
 };
 typedef struct parse_packet_ctx_s *parse_packet_ctx_t;
 
@@ -633,6 +634,7 @@ typedef struct parse_packet_ctx_s *parse_packet_ctx_t;
     (a)->last_pkt.pkt.generic= NULL;\
     (a)->free_last_pkt = 0;         \
     (a)->skip_meta = 0;             \
+    (a)->n_parsed_packets = 0;      \
   } while (0)
 
 #define deinit_parse_packet(a) do { \
